package com.javainuse.spring_boot_ai.elasticsearch;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class EmbeddingService {

	@Autowired
	private EmbeddingRepository embeddingRepository;

	/**
	 * Store a new embedding
	 */
	public EmbeddingDto storeEmbedding(EmbeddingDto embeddingDto) {
		if (embeddingDto.getId() == null || embeddingDto.getId().isEmpty()) {
			embeddingDto.setId(UUID.randomUUID().toString());
		}

		EmbeddingDocument document = mapToDocument(embeddingDto);
		EmbeddingDocument savedDocument = embeddingRepository.save(document);
		return mapToDto(savedDocument);
	}
	
	public List<EmbeddingDto> findSimilarDocuments(List<Double> queryVector) {
		List<EmbeddingDocument> similarDocuments = embeddingRepository.findSimilarDocuments(queryVector);
		return similarDocuments.stream().map(this::mapToDto).collect(Collectors.toList());
	}

	private EmbeddingDocument mapToDocument(EmbeddingDto dto) {
		return new EmbeddingDocument(dto.getId(), dto.getText(), dto.getEmbedding());
	}

	private EmbeddingDto mapToDto(EmbeddingDocument document) {
		return new EmbeddingDto(document.getId(), document.getText(), document.getEmbedding());
	}
}